// ======================================================================
//
// TranslucentQueue.h
// jeff grills
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef TRANSLUCENT_QUEUE_H
#define TRANSLUCENT_QUEUE_H

// ======================================================================

class  GL_abstract;
class  StaticMaterial;
struct TVertex;

// ======================================================================

class TranslucentQueue
{
public:

	typedef void (*DrawFunction)(const TVertex *vertexList, int numberOfVertices);

private:

	enum
	{
		PRIMITIVE_BUFFER_SIZE = 1000,
		VERTEX_BUFFER_SIZE    = PRIMITIVE_BUFFER_SIZE * 3,
	};

private:

	// disable the copy constructor and assignment operators
	TranslucentQueue(void);
	TranslucentQueue(const TranslucentQueue &);
	TranslucentQueue &operator =(const TranslucentQueue &);

private:

	struct Primitive
	{
		Primitive            *next;
		real                  sort;

		const StaticMaterial *material;

		DrawFunction          drawFunction;
		TVertex              *vertices;
		int                   numberOfVertices;
	};

private:

	static GL_abstract          *gl;

	static DrawFunction          pointDraw;
	static DrawFunction          lineDraw;
	static DrawFunction          triangleDraw;

	static Primitive            *firstSorted;

	static const StaticMaterial *material;

	static int                   vertexBufferCount;
	static int                   primitiveBufferCount;

	static TVertex               vertexBuffer[VERTEX_BUFFER_SIZE];
	static Primitive             primitiveBuffer[PRIMITIVE_BUFFER_SIZE];

private:

	static void sort(Primitive *primitive);

public:

	static void install(GL_abstract *gl, DrawFunction newPointDraw, DrawFunction newLineDraw, DrawFunction newTriangleDraw);
	static void remove(void);

	static void setMaterial(const StaticMaterial *newMaterial);

	static void addPoint(const TVertex *vertex);
	static void addLine(const TVertex *vertex0, const TVertex *vertex1);
	static void addTriangle(const TVertex *vertex0, const TVertex *vertex1, const TVertex *vertex2);

	static void draw(void);
};

// ======================================================================

inline void TranslucentQueue::setMaterial(const StaticMaterial *newMaterial)
{
	material = newMaterial;
}

// ======================================================================

#endif

